/***************************************************************************
 $RCSfile: kvkd.cpp,v $
 -------------------
 cvs         : $Id: kvkd.cpp,v 1.6 2003/02/09 01:59:26 aquamaniac Exp $
 begin       : Thu Dec 12 2002
 copyright   : (C) 2002 by Martin Preuss
 email       : martin@libchipcard.de

 ****************************************************************************
 * This program is free software; you can redistribute it and/or modify     *
 * it under the terms of the GNU General Public License as published by     *
 * the Free Software Foundation; either version 2 of the License, or        *
 * (at your option) any later version.                                      *
 *                                                                          *
 * This program is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU General Public License for more details.                             *
 *                                                                          *
 * You should have received a copy of the GNU General Public License        *
 * along with this program; if not, write to the Free Software              *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA *
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <chipcard.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <string>
using namespace std;

#ifdef HAVE_SIGNAL_H
# include <signal.h>
#endif



static int KVKDaemonStop=0;




#define k_PRG "kvkd"
#define k_PRG_VERSION_INFO \
  "kvkd v0.1  (part of libchipcard v"k_CHIPCARD_VERSION_STRING")\n"\
  "(c) 2002 Martin Preuss<martin@libchipcard.de>\n" \
  "This program is free software licensed under GPL.\n"\
  "See COPYING for details.\n"


void usage(string name) {
  fprintf(stderr,
	  "KVK Daemon - A daemon for German medical cards.\n"
	  "(c) 2002 Martin Preuss<martin@libchipcard.de>\n"
	  "This program is free software; you can redistribute it and/or\n"
	  "modify it under the terms of the GNU Lesser General Public\n"
	  "License as published by the Free Software Foundation; either\n"
	  "version 2.1 of the License, or (at your option) any later version.\n"
	  "\n"
	  "Usage:\n"
	  "%s [OPTIONS]\n"
	  " OPTIONS\n"
	  " -C CONFIGFILE - use the given LibChipCard configuration file\n"
	  "                 (defaults to "CHIPCARDC_CFGFILE"\n"
	  " -d DESTDIR    - destination folder for the card data read\n"
#ifdef HAVE_FORK
	  " -f            - start in foreground mode (otherwise fork into\n"
	  "                 background)\n"
#endif
	  ,
	  name.c_str());
}


struct s_args {
  string configFile;    // -C
  string destDir;       // -d
#ifdef HAVE_FORK
  bool daemonMode;      // -f
#endif
  list<string> params;
};


int checkArgs(s_args &args, int argc, char **argv) {
  int i;
  string tmp;

  i=1;
  args.configFile=CHIPCARDC_CFGFILE;
  args.destDir="/var/spool/gnumed";
#ifdef HAVE_FORK
  args.daemonMode=true;
#endif

  while (i<argc){
    tmp=argv[i];
    if (tmp=="-d") {
      i++;
      if (i>=argc)
	return 1;
      args.destDir=argv[i];
    }
    else if (tmp=="-C") {
      i++;
      if (i>=argc)
	return 1;
      args.configFile=argv[i];
    }
#ifdef HAVE_FORK
    else if (tmp=="-f") {
      args.daemonMode=false;
    }
#endif
    else if (tmp=="-h" || tmp=="--help") {
      usage(argv[0]);
      return -1;
    }
    else if (tmp=="-V" || tmp=="--version") {
      fprintf(stdout,k_PRG_VERSION_INFO);
      return -1;
    }
    else
      // otherwise add param
      args.params.push_back(tmp);
    i++;
  } // while
  // that's it
  return 0;
}


#ifdef HAVE_SIGACTION
/* Signal handler */

void signalHandler(int s) {
  switch(s) {
  case SIGINT:
    fprintf(stderr,"Got an interrupt signal\n");
    KVKDaemonStop=1;
    break;

  case SIGTERM:
    fprintf(stderr,"Got an termination signal\n");
    KVKDaemonStop=1;
    break;

#ifdef SIGINFO
  case SIGINFO:
    fprintf(stderr,"Got an info signal\n");
    break;
#endif

#ifdef SIGHUP
  case SIGHUP:
    fprintf(stderr,"Got a hangup signal\n");
    break;
#endif

  default:
    fprintf(stderr,"Unknown signal  %d",s);
    break;
  } /* switch */
}


struct sigaction saINT, saTERM, saINFO, saHUP;


int setSignalHandler() {

  saINT.sa_handler=signalHandler;
  sigemptyset(&saINT.sa_mask);
  saINT.sa_flags=0;
  if (sigaction(SIGINT, &saINT,0)) {
    fprintf(stderr,"Could not setup signal handler for SIGINT\n");
    return 2;
  }


  saTERM.sa_handler=signalHandler;
  sigemptyset(&saTERM.sa_mask);
  saTERM.sa_flags=0;
  if (sigaction(SIGTERM, &saTERM,0)) {
    fprintf(stderr,"Could not setup signal handler for SIGTERM\n");
    return 2;
  }

#ifdef SIGINFO
  saINFO.sa_handler=signalHandler;
  sigemptyset(&saINFO.sa_mask);
  saINFO.sa_flags=0;
  if (sigaction(SIGINT, &saINFO,0)) {
    fprintf(stderr,"Could not setup signal handler for SIGINFO\n");
    return 2;
  }
#endif

#ifdef SIGHUP
  saHUP.sa_handler=signalHandler;
  sigemptyset(&saHUP.sa_mask);
  saINT.sa_flags=0;
  if (sigaction(SIGHUP, &saHUP,0)) {
    fprintf(stderr,"Could not setup signal handler for SIGHUP\n");
    return 2;
  }
#endif

  return 0;
}
#endif



int handleCard(s_args &args, CTPointer<CTKVKCard> card){
  CTError err, err2;
  insuranceData idata;
  string filenamebase;
  string filename;
  string tmpname;
  FILE *f;
  int i;
  struct tm *ltime;
  time_t tms;

  fprintf(stderr,"Got a card.\n");
  err=card.ref().openCard();
  if (!err.isOk()) {
    fprintf(stderr, "Error in openCard: %s\n",
	    err.errorString().c_str());
    return 1;
  }
  err=card.ref().readCardData(idata);
  err2=card.ref().closeCard();
  if (!err.isOk()) {
    fprintf(stderr, "Error in readCardData: %s\n",
	    err.errorString().c_str());
    return 2;
  }
  if (!err2.isOk()) {
    fprintf(stderr, "Error in closeCard: %s\n",
	    err.errorString().c_str());
    return 3;
  }
  filenamebase=args.destDir;
  if (filenamebase.empty()) {
    fprintf(stderr,"Missing destination directory !\n");
    return 4;
  }
  if (filenamebase[filenamebase.length()-1]!='/')
    filenamebase+="/";

  // get current date and time
  tms=time(NULL);
  ltime=localtime(&tms);

  // try to create a unique filename
  i=10;
  while(i) {
    filename=filenamebase;
    filename+="KVK-";
#ifdef HAVE_RANDOM
    filename+=CTMisc::num2string(random());
#else
    filename+=CTMisc::num2string(rand());
#endif
    filename+=".dat";
    // check whether the file exists
    f=fopen(filename.c_str(),"r");
    if (f!=0)
      fclose(f);
    if (f==0)
      // found a free name
      break;
    i--;
  }
  if (!i) {
    fprintf(stderr,
	    "WARNING: Could not create a unique name within 10 tries !\n"
	    "Please report to bugs@libchipcard.de\n");
    return 5;
  }

  tmpname=filename+".tmp";
  f=fopen(tmpname.c_str(),"w+");
  if (f==0) {
    fprintf(stderr,
	    "Could not create file \"%s\", reason: \n %s\n",
	    tmpname.c_str(),strerror(errno));
    return 6;
  }

  // now print the data
  fprintf(f,"Version:libchipcard-"k_CHIPCARD_VERSION_STRING"\n");
  fprintf(f,"Datum:%02d.%02d.%04d\n",
	  ltime->tm_mday,
	  ltime->tm_mon,
	  ltime->tm_year+1900);
  fprintf(f,"Zeit:%02d:%02d:%02d\n",
	  ltime->tm_hour,
	  ltime->tm_min,
	  ltime->tm_sec);
  fprintf(f,"KK-Name:%s\n",idata.insuranceCompanyName.c_str());
  fprintf(f,"KK-Nummer:%s\n",idata.insuranceCompanyCode.c_str());
  fprintf(f,"KVK-Nummer:%s\n",idata.cardNumber.c_str());
  fprintf(f,"V-Nummer:%s\n",idata.insuranceNumber.c_str());
  fprintf(f,"V-Status:%s\n",idata.insuranceState.c_str());
  fprintf(f,"V-Statusergaenzung:%s\n",idata.eastOrWest.c_str());
  fprintf(f,"Titel:%s\n",idata.title.c_str());
  fprintf(f,"Vorname:%s\n",idata.forename.c_str());
  fprintf(f,"Namenszusatz:%s\n",idata.nameSuffix.c_str());
  fprintf(f,"Familienname:%s\n",idata.name.c_str());
  fprintf(f,"Geburtsdatum:%s\n",idata.dateOfBirth.c_str());
  fprintf(f,"Strasse:%s\n",idata.addrStreet.c_str());
  fprintf(f,"Laendercode:%s\n",idata.addrState.c_str());
  fprintf(f,"PLZ:%s\n",idata.addrPostalCode.c_str());
  fprintf(f,"Ort:%s\n",idata.addrCity.c_str());
  fprintf(f,"gueltig-bis:%s\n",idata.bestBefore.c_str());
  fprintf(f,"Pruefsumme-gueltig:%s\n",
	  idata.isValid?"ja":"nein");
  fprintf(f,"Kommentar:derzeit keiner\n");
  if (fclose(f)) {
    fprintf(stderr,
	    "Could not close file \"%s\", reason: \n %s\n",
	    tmpname.c_str(),strerror(errno));
    return 7;
  }
  // finally move the file
  if (rename(tmpname.c_str(),filename.c_str())) {
    fprintf(stderr,
	    "Could not rename file \"%s\" to \"%s\", reason: \n %s\n",
	    tmpname.c_str(),filename.c_str(),strerror(errno));
    return 8;
  }
  fprintf(stderr,"Card done.\n");
  return 0;
}


int main(int argc, char **argv) {
  int rv;
  CTPointer<CTKVKCard> card;
  s_args args;
  string cmd;
  int requestid;
  unsigned int rflags;
  unsigned int rstatus;

  rv=checkArgs(args,argc,argv);
  if (rv==-1)
    return 0;
  else if (rv)
    return rv;

#ifdef HAVE_FORK
  if (args.daemonMode) {
    rv=fork();
    if (rv==-1) {
      fprintf(stderr,"Error on fork, aborting.\n");
      return 2;
    }
    else if (rv!=0) {
      fprintf(stderr,"Daemon forked, father exiting.\n");
      return 0;
    }
  }
#endif

#ifdef HAVE_SIGACTION
  if (setSignalHandler()) {
    fprintf(stderr,"Error setting up signal handler, aborting\n");
    return 2;
  }
#endif

  /* setup random number generator
   * Note: The check for HAVE_RANDOM is ok, because I think it's better to
   * use matching function calls (random-srandom / rand/srand)
   */
#ifdef HAVE_RANDOM 
  srandom(time(NULL));
#else
  srand(time(NULL));
#endif

  rv=ChipCard_Init(args.configFile.c_str(),0);
  if (rv!=CHIPCARD_SUCCESS) {
    fprintf(stderr,"Could not init libchipcard (%d)\n",rv);
    return 2;
  }

  /* create new instance of the KVK card */
  card=new CTKVKCard();

  rv=card.ref().startWaitForCard(requestid,
				 true,
				 0,0,
				 CHIPCARD_STATUS_INSERTED,
				 CHIPCARD_STATUS_INSERTED |
				 CHIPCARD_STATUS_PROCESSOR|
				 CHIPCARD_STATUS_LOCKED_BY_OTHER);
  if (rv!=CHIPCARD_SUCCESS) {
    fprintf(stderr,"WARNING: Could not issue request (Err=%d)\n",rv);
    return 2;
  }


  while(KVKDaemonStop==0) {
    rv=card.ref().checkWaitForCard(requestid,
				   30,
				   rflags,
				   rstatus);
    if (rv==CHIPCARD_SUCCESS) {
      // got a card, read it
      rv=handleCard(args, card);
      if (rv) {
	fprintf(stderr, "Error handling card (%d)\n",rv);
      }
    }
    else if (rv!=CHIPCARD_ERROR_NO_MESSAGE) {
      fprintf(stderr,"Error while waiting for card (%d)\n",rv);
    }
  } // while


  fprintf(stderr,"Stopping KVK daemon.\n");
  card=0;
  fprintf(stderr,"KVK daemon stopped.\n");
  return 0;
}








