/***************************************************************************
 $RCSfile: tutorial1.cpp,v $
                             -------------------
    cvs         : $Id: tutorial1.cpp,v 1.3 2002/12/10 18:48:34 aquamaniac Exp $
    begin       : Sun Nov 18 2001
    copyright   : (C) 2001 by Martin Preuss
    email       : martin@libchipcard.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

/*
 * This is a small tutorial on how to use the basic functions of the
 * libchipcard. It just determines whether an inserted card is a processor
 * or a memory card.
 * Usage:
 *   tutorial1
 */


#include <stdio.h>

// you need only the following include
#include "chipcard.h"


int main(int argc,char **argv) {
  /* This variable receives the error codes on calls to libchipcard */
  CTError err;

  // pointer to the created chip card object
  CTCard *card;

  int rv;
  unsigned int readerFlags;
  unsigned int status;

  /* Initialize LibChipCard. This must be done once in your program.
   * However, you may even call this method as often as you like, only the
   * first call will actually perform the initialization ;-)
   */
  rv=ChipCard_Init(0,0);
  if (rv!=CHIPCARD_SUCCESS) {
    fprintf(stderr,"Sorry, could not initialize LibChipCard (%d)\n",
	    rv);
    return 2;
  }

  /* Ok, now we instantiate a CTCard object.
   * An object of this class may be instantiated even if there is no
   * card in the reader, or even worse, if you DON'T have a reader ;-)
   */
  card=new CTCard();

  /* now we wait for the user to insert a chip card in any reader
   * The second argument might be a bit confusing: If true, then the method
   * waits for the NEXT card, ignoring the currently inserted one.
   * You can use this feature to check another card after you checked the
   * currently inserted card. False means that we would like to use even
   * a currently inserted card or wait if there is none.
   */
  fprintf(stderr,"Please insert your card.\n");
  readerFlags=0;
  status=CHIPCARD_STATUS_INSERTED;
  if (!card->waitForCard(1000,  // timeout in seconds
			 false, // use an already inserted card, if any
			 readerFlags,  // readerFlags (ignored here)
			 0,     // mask for readerFlags (ignored here)
			 status,
			 CHIPCARD_STATUS_INSERTED)) {
    fprintf(stderr,"No card inserted within some seconds, aborting");
    return 2;
  }
  fprintf(stderr,"Card inserted, working.\n");

  /* Now we get in contact with the chip card. When calling this method
   * the light of the reader should go on and the card is powered on.
   * If this method successfully returns the card is accessable.
   */
  err=card->openCard();
  // the following is an example of a complete error checking
  if (!err.isOk()) {
    printf("ERROR: Opening card. (%s)\n",
	   err.errorString().c_str());
    /* deinitialize LibChipCard */
    ChipCard_Fini();
    return 1;
  }

  // check if it is a processor or memory card
  if (card->isProcessorCard())
    printf("The card is a processor card.\n");
  else {
    printf("The card is a memory card (size=%d).\n",
	   card->memorySize());
  }
  /* After calling this the light of the card reader should go off and
   * the chip card is powered down. Now you may remove the card.
   * For some reasons I do not know the terminal returns a status of
   * 0x62 upon disconnecting a chip card, so I take this value for an ok.
   * UPDATE: The code 0x62 means that no card is connected, which is exactly
   * what we want from this method ;-)
   */
  err=card->closeCard();
  /* we allow the error code 0x62 to be valid, since this code shows
   * that the card is disconnected, which was the intention of closing
   * the card. Hence the parameter 0x62 */
  if (!err.isOk(0x62)) {
    printf("ERROR: Closing card. (%s)\n",
	   err.errorString().c_str());
    /* deinitialize LibChipCard */
    ChipCard_Fini();
    return 1;
  }

  /* deinitialize LibChipCard */
  ChipCard_Fini();

  printf("Bye.\n");
  return 0;
}







